#include "i2c_master.h"

void i2c_master_init(void)
{
//	twi_init(TWI_RATE_20K);  // I2C clock == 20kHz
	twi_init(TWI_RATE_400K); // I2C clock == 400kHz
}

int i2c_start(int address, int rw)
{
	int ret;
	ret = sla_set( address + rw ); // rw==1 ... Read  /  rw==0 ... Write
	if (ret==0x18) { return 0; } // ACK (Write)
	if (ret==0x40) { return 0; } // ACK (Read)
	return 1;	 // 1 ... Error: no ACK or some other error

}

void i2c_write(int data)
{
	TWI_write(data);
}

int i2c_read(int ack_mode)
{
	return TWI_read(ack_mode);  // ack_mode==1 ... read with ACK  /  ack_mode==0 ... with NACK
}

void i2c_stop()
{
	TWI_STOP;
}

/*********************  TWI Rate Table  *********************/
const char twi_tbl[] = {2,30 ,1,48, 0,92,0,17,0,6 }; //20khz,50khz,100khz,400khz,650khz

void twi_init(unsigned char twi_rate)
{
	TWCR = 0;           // TWI Disable
	 TWSR = twi_tbl[twi_rate * 2];       // TWI prescaler
	 TWBR = twi_tbl[twi_rate * 2+1];     // TWBR setting
	TWCR = _BV(TWEN);   // TWI Enable
}


uint8_t TWI_wait(void)
{
	int8_t r;
	int retry_cu = 0;
	while (1) {
		// wait TWCR:TWINT bit --> High
		if ((TWCR & 0x80) == 0x80) { break; }
		retry_cu++;
		if (retry_cu > 500) { break; } // to much retry count
	}
	r = TW_STATUS;
	return r;
}


char sla_set(uint8_t sla_rw)
{
	char r;
	TWI_START;
	r = TWI_wait();
	if ((r != TW_REP_START)&&(r != TW_START)) { return r; }
	r = TWI_write(sla_rw);
	if (r == TW_MT_ARB_LOST) { return r; }
	if (r == TW_MT_SLA_ACK)  { return r; } // if OK,return TW_MT_SLA_ACK.
	if (r == TW_MR_SLA_ACK)  { return r; }
	TWI_STOP;
	return r;
}


uint8_t TWI_write(uint8_t x)
{
	TWDR = x;
	TWI_NEXT;
	return TWI_wait();
}

uint8_t TWI_read(char ack)
{
	uint8_t r;
	if (ack) { TWI_ACK;  }
	else {     TWI_NACK; }
	TWI_wait();
	r = TWDR;
	return r;
}
