#include <avr/io.h>
#include <avr/interrupt.h>
#include "uart.h"

// AVR}CRH̃NbN̎gŎw肷!
#define CLOCK 8000000UL  // 8MHz
//#define CLOCK 16000000UL  // 16MHz

typedef unsigned char byte;
volatile byte uart_recv_buf[256]; // UART Receive buffer
volatile byte uart_recv_buf_write_pos,uart_recv_buf_read_pos;

//ISR(USART0_RX_vect)
ISR(USART_RX_vect)
{
	// UART receive interrupt  (data received)
	uart_recv_buf[uart_recv_buf_write_pos++] = UDR0; // receive to buffer
}

void rs_init_by_BRR(unsigned int brr)
{
	UBRR0H = (unsigned char)(brr>>8); UBRR0L = (unsigned char)brr;
	// stopbit 1bit , data=8bit
	UCSR0C = (3<<UCSZ00);

	// RX,TX enable  , RX interrupt enable
	UCSR0B = (1<<RXEN0) | (1<<TXEN0) | (1<<RXCIE0);
	
	sei();
}

void rs_init_by_BRR_2x(unsigned int brr)
{
    UCSR0A = 0x02;  // 2x
	rs_init_by_BRR(brr);
}

void rs_init_1x(unsigned long baud)
{
	unsigned int brr; // baud rate reg
	brr = (CLOCK>>4)/baud - 1;
	rs_init_by_BRR(brr);
}

void rs_init_2x(unsigned long baud)
{
	unsigned int brr; // baud rate reg
	brr = (CLOCK>>3)/baud - 1;
	rs_init_by_BRR_2x(brr);
}


void rs_init(unsigned long baud)
{
	rs_init_2x(baud);
}

int rs_size(void)
{
	if (uart_recv_buf_write_pos ==  uart_recv_buf_read_pos) {
		return 0;
	}
	return 1;
}

int rs_getc()
{
	while (1) {
		if (rs_size()>0) { break; }
	}
    return uart_recv_buf[uart_recv_buf_read_pos++];
}

void rs_putc(int c)
{
	while (1) {
		if (UCSR0A & (1<<UDRE0)) { break; }	// possible to send?
	}
	UDR0 = c;
}

void rs_puts(char *s)
{
	while (*s) { rs_putc(*s++); }
}
